//------------------------------------------------------------------------------
//
// File:    DGraph_impl.h
//
// Purpose: Directed graph implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "DEdge.h"
#include "DVertex.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
void DGraph<TEdge,TVertex> :: addEdge( TEdge * edge )
{
  TVertex * dvertex1;
  TVertex * dvertex2;

  Graph<TEdge,TVertex> :: addEdge( edge );

  dvertex1 = edge->getFromDVertex();
  dvertex1->addFromDEdge( edge );

  dvertex2 = edge->getToDVertex();

  dvertex2->addToDEdge( edge );
}

template <class TEdge,class TVertex>
void DGraph<TEdge,TVertex> :: addVertex( TVertex * vertex )
{
  Graph<TEdge,TVertex> :: addVertex( vertex );
}

template <class TEdge,class TVertex>
TEdge * DGraph<TEdge,TVertex> :: findEdge( const TVertex * fromVertex, const TVertex * toVertex ) const
{
  TEdge         * dedge;
  const TVertex * dvertex;

  dvertex = fromVertex;

  dedge = dvertex->getFirstFromDEdge();

  while( dedge ) {
    if( dedge->getToDVertex() == toVertex ) {
      return( dedge );
    }

    dedge = dedge->getNextFromDEdge();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
void DGraph<TEdge,TVertex> :: removeEdge( TEdge * edge )
{
  TVertex * vertex;
  TEdge   * dedge;

  dedge = edge;

  vertex = dedge->getFromDVertex();

  vertex->removeFromEdge( dedge );

  vertex = dedge->getToDVertex();

  vertex->removeToEdge( dedge );

  Graph<TEdge,TVertex> :: removeEdge( dedge );
}

template <class TEdge,class TVertex>
void DGraph<TEdge,TVertex> :: removeVertex( TVertex * vertex )
{
  TVertex * dvertex = vertex;
  TEdge   * dedge   = dvertex->getFirstFromDEdge();

  while( dedge ) {
    removeEdge( dedge );

    dedge = dvertex->getFirstFromDEdge();
  }

  dedge = dvertex->getFirstToDEdge();

  while( dedge ) {
    removeEdge( dedge );

    dedge = dvertex->getFirstToDEdge();
  }

  Graph<TEdge,TVertex> :: removeVertex( dvertex );
}

template <class TEdge,class TVertex>
void DGraph<TEdge,TVertex> :: deleteAllEdges( void )
{
  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    vertex->clearAllEdges();

    vertex = vertex->getNext();
  }

  Graph<TEdge,TVertex> :: deleteAllEdges();
}

//----------------------- DGraph check staff -------------------------

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: findEdgeInVertex( TEdge * edge ) const
{
  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    if( vertex->IfEdgeInVertex( edge ) ) {
      return( 1 );
    }

    vertex = vertex->getNext();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: findEdgeInGraph( TEdge * edge ) const
{
  TEdge * edge1 = Graph<TEdge,TVertex>::getFirstEdge();

  while( edge1 ) {
    if( edge == edge1 ) {
      return( 1 );
    }

    edge1 = edge1->getNext();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: CheckEdgesInVertex( TVertex * vertex ) const
{
  TEdge * edge = vertex->getFirstFromDEdge();

  while( edge ) {
    if( !findEdgeInGraph( edge ) ) {
      return( 1 );
    }

    edge = edge->getNextFromDEdge();
  }

  edge = vertex->getFirstToDEdge();

  while( edge ) {
    if( !findEdgeInGraph( edge ) ) {
      return( 1 );
    }

    edge = edge->getNextToDEdge();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: CheckEdgesConsistency( void ) const
{
  TEdge * edge = Graph<TEdge,TVertex>::getFirstEdge();

  while( edge ) {
    if( !findEdgeInVertex( edge ) ) {
      return( 1 );
    }

    edge = edge->getNext();
  }

  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    if( CheckEdgesInVertex( vertex ) ) {
      return( 2 );
    }

    vertex = vertex->getNext();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: getEdgesInNodesCount( void ) const
{
  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();
  int       count  = 0;

  while( vertex ) {
    count = count + vertex->getEdgesCount();

    vertex = vertex->getNext();
  }

  return( count );
}

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: checkGraph( void ) const
{
  int retCode = Graph<TEdge,TVertex> :: checkGraph();

  if( retCode ) {
    return( retCode );
  }

  if( 2 * Graph<TEdge,TVertex>::getEdgesCount() != getEdgesInNodesCount() ) {
    return( 30 );
  }

  if( CheckEdgesConsistency() ) {
    return( 31 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
std::ostream & DGraph<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  s << std::endl << "Directed graph: " << std::endl;

  Graph<TEdge,TVertex> :: Print( s, Mod );

  return( s );
}

//----------- Algorithms ----------------

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: recFindPath( TVertex * fromVertex, TVertex * toVertex ) const
{
  if( fromVertex == toVertex ) {
    return( 1 );
  }

  fromVertex->setVisit( 1 );

  TEdge * edge = fromVertex->getFirstFromDEdge();

  while( edge ) {
    TVertex * vertex = edge->getToDVertex();

    if( !vertex->getVisit() ) {
      if( recFindPath( vertex, toVertex ) ) {
        return( 1 );
      }
    }

    edge = edge->getNextFromDEdge();
  }  

  return( 0 );
}

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: findPath( TVertex * fromVertex, TVertex * toVertex ) const
{
  Graph<TEdge,TVertex>::clearAllVisitFlags();

  return( recFindPath( fromVertex, toVertex ) );
}

//------------------------ Reingold FindAllCycles ---------------------------

template <class TEdge,class TVertex>
int DGraph<TEdge,TVertex> :: FindAllCycles( std::list<std::list<GraphElem*> > & Cycles )
{
  std::stack<GraphElem*> stack;

  TVertex * sourceVertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( sourceVertex ) {
    TVertex * tmpVertex = sourceVertex->getNext();

    while( tmpVertex ) {
      tmpVertex->setTmpFlag( 1 );

      TEdge * tmpEdge = tmpVertex->getFirstToDEdge();

      while( tmpEdge ) {
        tmpEdge->setTmpFlag( 0 );

        tmpEdge = tmpEdge->getNextToDEdge();
      }

      tmpVertex = tmpVertex->getNext();
    }

    FindCycle( sourceVertex, sourceVertex, stack, Cycles );

    sourceVertex->setTmpFlag( 0 );

    sourceVertex = sourceVertex->getNext();
  }

  return( Cycles.size() );
}

//-------------------------- Reingold FindCycle -----------------------------

template <class TEdge,class TVertex>
bool DGraph<TEdge,TVertex> :: FindCycle( TVertex * vertex, TVertex * sourceVertex,
                                         std::stack<GraphElem*> & stack,
                                         std::list<std::list<GraphElem*> > & Cycles )
{
  bool PartOfCycle = false;
 
  stack.push( vertex );
  vertex->setTmpFlag( 0 );

  TEdge * tmpEdge = vertex->getFirstFromDEdge();

  while( tmpEdge ) {
    TVertex * tmpVertex = tmpEdge->getToDVertex();

    if( vertex != tmpVertex ) {
      stack.push( tmpEdge );

      if( tmpVertex->getTmpFlag() ) {
        if( FindCycle( tmpVertex, sourceVertex, stack, Cycles ) ) {
          PartOfCycle = true;
        }
      }
      else { 
        if( tmpVertex == sourceVertex ) { // If node equal to source node - cycle was found
          PartOfCycle = true;

          std::list<GraphElem*> list;
          std::stack<GraphElem*> tmpStack = stack;

          while( !tmpStack.empty() ) {
            list.push_front( tmpStack.top() );

            tmpStack.pop();
          }

          Cycles.push_back( list );
        }
      }

      stack.pop();  // Pop edge
    }

    tmpEdge = tmpEdge->getNextFromDEdge();
  }

  stack.pop();  // Pop vertex

  if( PartOfCycle ) {
    Unmark( vertex );
  }
  else {
    TEdge * tmpEdge = vertex->getFirstFromDEdge();

    while( tmpEdge ) {
      tmpEdge->setTmpFlag( 1 );

      tmpEdge = tmpEdge->getNextFromDEdge();
    }
  }

  return( PartOfCycle );
}

//================================ Unmark ====================================

template <class TEdge,class TVertex>
void DGraph<TEdge,TVertex> :: Unmark( TVertex * vertex )
{
  vertex->setTmpFlag( 1 );

  TEdge * tmpEdge = vertex->getFirstToDEdge();

  while( tmpEdge ) {
    if( tmpEdge->getTmpFlag() ) {
      TVertex * tmpVertex = tmpEdge->getFromDVertex();

      tmpEdge->setTmpFlag( 0 );

      if( !tmpVertex->getTmpFlag() ) {
        Unmark( tmpVertex );
      }
    }

    tmpEdge = tmpEdge->getNextToDEdge();
  }
}

}}

//---------------------------- End of file ---------------------------
