//------------------------------------------------------------------------------
//
// File:    DVertex_impl.h
//
// Purpose: Directed vertex implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "DEdge.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
void DVertex<TEdge,TVertex> :: addFromDEdge( TEdge * edge )
{
  if( lastFromDEdge ) {
    edge->nextFromDEdge          = 0;
    edge->prevFromDEdge          = lastFromDEdge;
    lastFromDEdge->nextFromDEdge = edge;
    lastFromDEdge                = edge;
  }
  else {
    edge->nextFromDEdge = 0;
    edge->prevFromDEdge = 0;
    firstFromDEdge      = edge;
    lastFromDEdge       = edge;
  }
}

template <class TEdge,class TVertex>
void DVertex<TEdge,TVertex> :: addToDEdge( TEdge * edge )
{
  if( lastToDEdge ) {
    edge->nextToDEdge        = 0;
    edge->prevToDEdge        = lastToDEdge;
    lastToDEdge->nextToDEdge = edge;
    lastToDEdge              = edge;
  }
  else {
    edge->nextToDEdge = 0;
    edge->prevToDEdge = 0;
    firstToDEdge      = edge;
    lastToDEdge       = edge;
  }
}

template <class TEdge,class TVertex>
void DVertex<TEdge,TVertex> :: removeFromEdge( TEdge * edge )
{
  TEdge * edgePrev = edge->getPrevFromDEdge();
  TEdge * edgeNext = edge->getNextFromDEdge();

  if( edgePrev ) {
    edgePrev->nextFromDEdge = edgeNext;
  }
  else {
    firstFromDEdge = edgeNext;
  }

  if( edgeNext ) {
    edgeNext->prevFromDEdge = edgePrev;
  }
  else {
    lastFromDEdge = edgePrev;
  }
}

template <class TEdge,class TVertex>
void DVertex<TEdge,TVertex> :: removeToEdge( TEdge * edge )
{
  TEdge * edgePrev = edge->getPrevToDEdge();
  TEdge * edgeNext = edge->getNextToDEdge();

  if( edgePrev ) {
    edgePrev->nextToDEdge = edgeNext;
  }
  else {
    firstToDEdge = edgeNext;
  }

  if( edgeNext ) {
    edgeNext->prevToDEdge = edgePrev;
  }
  else {
    lastToDEdge = edgePrev;
  }
}

template <class TEdge,class TVertex>
void DVertex<TEdge,TVertex> :: clearAllEdges( void )
{
  firstFromDEdge = 0;
  lastFromDEdge  = 0;
  firstToDEdge   = 0;
  lastToDEdge    = 0;
}

template <class TEdge,class TVertex>
int DVertex<TEdge,TVertex> :: getFromEdgesCount( void ) const
{
  int     count = 0;
  TEdge * edge  = firstFromDEdge;

  while( edge ) {
    count++;

    edge = edge->getNextFromDEdge();
  }

  return( count );
}

template <class TEdge,class TVertex>
int DVertex<TEdge,TVertex> :: getToEdgesCount( void ) const
{
  int     count = 0;
  TEdge * edge  = firstToDEdge;

  while( edge ) {
    count++;

    edge = edge->getNextToDEdge();
  }

  return( count );
}

template <class TEdge,class TVertex>
int DVertex<TEdge,TVertex> :: getEdgesCount( void ) const
{
  return( getFromEdgesCount() + getToEdgesCount() );
}

template <class TEdge,class TVertex>
int DVertex<TEdge,TVertex> :: IfEdgeInVertex( TEdge * edge ) const
{
  TEdge * edge1  = firstFromDEdge;

  while( edge1 ) {
    if( edge1 == edge ) {
      return( 1 );
    }

    edge1 = edge1->getNextFromDEdge();
  }

  edge1  = firstToDEdge;

  while( edge1 ) {
    if( edge1 == edge ) {
      return( 1 );
    }

    edge1 = edge1->getNextToDEdge();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int DVertex<TEdge,TVertex> :: check( void ) const
{
  TEdge   * edge;
  TEdge   * edgePrev;
  TEdge   * edgeNext;

  if( ( firstFromDEdge && !lastFromDEdge ) || ( !firstFromDEdge && lastFromDEdge ) ) {
    return( 1 );
  }

  if( ( firstToDEdge && !lastToDEdge ) || ( !firstToDEdge && lastToDEdge ) ) {
    return( 2 );
  }

  edge = firstFromDEdge;

  if( edge ) {
    if( edge->getPrevFromDEdge() ) {
      return( 3 );
    }

    edgePrev = edge;
    edge     = edge->getNextFromDEdge(); 

    while( edge ) {
      if( edge->getPrevFromDEdge() != edgePrev ) {
        return( 4 );
      }

      edgePrev = edge;
      edge = edge->getNextFromDEdge();
    }
  }

  edge = lastFromDEdge;

  if( edge ) {
    if( edge->getNextFromDEdge() ) {
      return( 5 );
    }

    edgeNext = edge;
    edge     = edge->getPrevFromDEdge(); 

    while( edge ) {
      if( edge->getNextFromDEdge() != edgeNext ) {
        return( 6 );
      }

      edgeNext = edge;
      edge = edge->getPrevFromDEdge();
    }
  }

  edge = firstToDEdge;

  if( edge ) {
    if( edge->getPrevToDEdge() ) {
      return( 7 );
    }

    edgePrev = edge;
    edge     = edge->getNextToDEdge(); 

    while( edge ) {
      if( edge->getPrevToDEdge() != edgePrev ) {
        return( 8 );
      }

      edgePrev = edge;
      edge = edge->getNextToDEdge();
    }
  }

  edge = lastToDEdge;

  if( edge ) {
    if( edge->getNextToDEdge() ) {
      return( 9 );
    }

    edgeNext = edge;
    edge     = edge->getPrevToDEdge(); 

    while( edge ) {
      if( edge->getNextToDEdge() != edgeNext ) {
        return( 10 );
      }

      edgeNext = edge;
      edge = edge->getPrevToDEdge();
    }
  }

  return( 0 );
}

template <class TEdge,class TVertex>
std::ostream & DVertex<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  TEdge   * edge;

  s << " Dir vertex: " << this << " Visited = " << Vertex<TEdge,TVertex>::getVisit()
    << " Tmp = " << GraphElem::getTmpFlag()
    << " FromEdges = " << getFromEdgesCount()
    << " ToEdges = " << getToEdgesCount() << std::endl;

  if( Mod > 0 ) {
    s << " From edges:" << std::endl;

    edge = firstFromDEdge;

    while( edge ) {
      edge->Print( s, Mod - 1 );

      edge = edge->getNextFromDEdge();
    }

    s << " To edges:" << std::endl;

    edge = firstToDEdge;

    while( edge ) {
      edge->Print( s, Mod - 1 );

      edge = edge->getNextToDEdge();
    }
  }

  return( s );
}

}}

//---------------------------- End of file ---------------------------
