//------------------------------------------------------------------------------
//
// File:    Graph_impl.h
//
// Purpose: Graph class implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "Edge.h"
#include "Vertex.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
Graph<TEdge,TVertex> :: ~Graph( void )
{
  clearGraph();
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: addEdge( TEdge * edge )
{
  if( lastEdge ) {
    edge->next     = 0;
    edge->prev     = lastEdge;
    lastEdge->next = edge;
    lastEdge       = edge;
  }
  else {
    edge->next = 0;
    edge->prev = 0;
    firstEdge  = edge;
    lastEdge   = edge;
  }
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: addVertex( TVertex * vertex )
{
  if( lastVertex ) {
    vertex->next     = 0;
    vertex->prev     = lastVertex;
    lastVertex->next = vertex;
    lastVertex       = vertex;
  }
  else {
    vertex->next = 0;
    vertex->prev = 0;
    firstVertex  = vertex;
    lastVertex   = vertex;
  }
}

template <class TEdge,class TVertex>
int Graph<TEdge,TVertex> :: getEdgesCount( void ) const
{
  int     count = 0;
  TEdge * edge  = firstEdge;

  while( edge ) {
    count++;

    edge = edge->next;
  }

  return( count );
}

template <class TEdge,class TVertex>
int Graph<TEdge,TVertex> :: getVerticesCount( void ) const
{
  int       count  = 0;
  TVertex * vertex = firstVertex;

  while( vertex ) {
    count++;

    vertex = vertex->next;
  }

  return( count );
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: removeEdge( TEdge * edge )
{
  TEdge * edgePrev = edge->prev;
  TEdge * edgeNext = edge->next;

  if( edgePrev ) {
    edgePrev->next = edgeNext;
  }
  else {
    firstEdge = edgeNext;
  }

  if( edgeNext ) {
    edgeNext->prev = edgePrev;
  }
  else {
    lastEdge = edgePrev;
  }

  edge->prev = 0;
  edge->next = 0;

  delete edge;
}

template <class TEdge,class TVertex>
int Graph<TEdge,TVertex> :: removeEdgeVertex( TVertex * fromVertex, TVertex * toVertex )
{
  TEdge * edge;
  int     retCode = 0;

  edge = findEdge( fromVertex, toVertex );

  if( edge ) {
    removeEdge( edge );
  }
  else {
    retCode = 1;
  }

  return( retCode );
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: removeVertex( TVertex * vertex )
{
  TVertex * vertexPrev = vertex->prev;
  TVertex * vertexNext = vertex->next;

  if( vertexPrev ) {
    vertexPrev->next = vertexNext;
  }
  else {
    firstVertex = vertexNext;
  }

  if( vertexNext ) {
    vertexNext->prev = vertexPrev;
  }
  else {
    lastVertex = vertexPrev;
  }

  vertex->prev = 0;
  vertex->next = 0;

  delete vertex;
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: clearGraph( void )
{
  TVertex * vertex = firstVertex;
  TVertex * vertexNext;

  deleteAllEdges();

  while( vertex ) {
    vertexNext = vertex->next;

    vertex->prev = 0;
    vertex->next = 0;

    delete vertex;

    vertex = vertexNext;
  }

  firstVertex = 0;
  lastVertex  = 0;
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: deleteAllEdges( void )
{
  TEdge * edge = firstEdge;
  TEdge * edgeNext;

  while( edge ) {
    edgeNext = edge->next;

    edge->prev = 0;
    edge->next = 0;

    delete edge;

    edge = edgeNext;
  }

  firstEdge = 0;
  lastEdge  = 0;
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: clearAllEdgesTmpFlags( void ) const
{
  TEdge * edge  = firstEdge;

  while( edge ) {
    edge->clearTmpFlag();

    edge = edge->next;
  }
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: clearAllVerticesTmpFlags( void ) const
{
  TVertex * vertex = firstVertex;

  while( vertex ) {
    vertex->clearTmpFlag();

    vertex = vertex->next;
  }
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: clearAllTmpFlags( void ) const
{
  clearAllEdgesTmpFlags();
  clearAllVerticesTmpFlags();
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: clearAllVisitFlags( void ) const
{
  TVertex * vertex = firstVertex;

  while( vertex ) {
    vertex->clearVisit();

    vertex = vertex->next;
  }
}

template <class TEdge,class TVertex>
int Graph<TEdge,TVertex> :: checkGraph( void ) const
{
  TEdge   * edge;
  TEdge   * edgePrev;
  TEdge   * edgeNext;
  TVertex * vertex;
  TVertex * vertexPrev;
  TVertex * vertexNext;

  if( ( firstEdge && !lastEdge ) || ( !firstEdge && lastEdge ) ) {
    graphError( 1, "Inconsistency firstEdge and lastEdge\n");
    return( 1 );
  }

  if( ( firstVertex && !lastVertex ) || ( !firstVertex && lastVertex ) ) {
    graphError( 2, "Inconsistency firstVertex and lastVertex\n");
    return( 2 );
  }

  edge = firstEdge;

  if( edge ) {
    if( edge->prev ) {
      graphError( 3, "Inconsistency in first edge\n");
      return( 3 );
    }

    edgePrev = edge;
    edge     = edge->next; 

    while( edge ) {
      if( edge->prev != edgePrev ) {
        graphError( 4, "Inconsistency in direct edge chain\n");
        return( 4 );
      }

      edgePrev = edge;
      edge = edge->next;
    }
  }

  edge = lastEdge;

  if( edge ) {
    if( edge->next ) {
      graphError( 5, "Inconsistency in last edge\n");
      return( 5 );
    }

    edgeNext = edge;
    edge     = edge->prev; 

    while( edge ) {
      if( edge->next != edgeNext ) {
        graphError( 6, "Inconsistency in reverse edge chain\n");
        return( 6 );
      }

      edgeNext = edge;
      edge = edge->prev;
    }
  }

  vertex = firstVertex;

  if( vertex ) {
    if( vertex->prev ) {
      graphError( 7, "Inconsistency in first vertex\n");
      return( 7 );
    }

    vertexPrev = vertex;
     vertex    = vertex->next; 

    while( vertex ) {
      if( vertex->prev != vertexPrev ) {
        graphError( 8, "Inconsistency in direct vertex chain\n");
        return( 8 );
      }

      vertexPrev = vertex;
      vertex = vertex->next;
    }
  }

  vertex = lastVertex;

  if( vertex ) {
    if( vertex->next ) {
      graphError( 9, "Inconsistency in last vertex\n");
      return( 9 );
    }

    vertexNext = vertex;
    vertex     = vertex->prev; 

    while( vertex ) {
      if( vertex->next != vertexNext ) {
        graphError( 10, "Inconsistency in reverse vertex chain\n");
        return( 10 );
      }

      vertexNext = vertex;
      vertex = vertex->prev;
    }
  }

  edge  = firstEdge;

  while( edge ) {
    if( edge->check() ) {
      return( 11 );
    }

    edge = edge->next;
  }

  vertex = firstVertex;

  while( vertex ) {
    if( vertex->check() ) {
      return( 12 );
    }

    vertex = vertex->next;
  }

  return( 0 );
}

template <class TEdge,class TVertex>
void Graph<TEdge,TVertex> :: graphError( const int Mod, const std::string & Msg ) const
{
  std::cerr << "ERROR " << Mod << ": " << Msg << std::endl;
}

template <class TEdge,class TVertex>
std::ostream & Graph<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  s << " Edges = " << getEdgesCount() << " Vertices = " << getVerticesCount() << std::endl;

  if( Mod > 0 ) {
    s << std::endl << "Edges:" << std::endl;

    TEdge * edge  = firstEdge;

    while( edge ) {
      edge->Print( s, Mod - 1 );

      edge = edge->next;
    }

    s << std::endl << "Vertices:" << std::endl;

    TVertex * vertex = firstVertex;

    while( vertex ) {
      vertex->Print( s, Mod - 1 );

      vertex = vertex->next;
    }
  }

  return( s );
}

}}

//---------------------------- End of file ---------------------------
