//------------------------------------------------------------------------------
//
// File:    UGraph_impl.h
//
// Purpose: Undirected graph implementation for graph library
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 1.0 30.04.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "UEdge.h"
#include "UVertex.h"

namespace aps {

  namespace apsgraph {

template <class TEdge,class TVertex>
void UGraph<TEdge,TVertex> :: addEdge( TEdge * edge )
{
  TVertex * uvertex1;
  TVertex * uvertex2;

  Graph<TEdge,TVertex> :: addEdge( edge );

  uvertex1 = edge->getUVertex1();
  uvertex1->addEdge1( edge );

  uvertex2 = edge->getUVertex2();

  if( uvertex1 != uvertex2 ) {
    uvertex2->addEdge2( edge );
  }
}

template <class TEdge,class TVertex>
void UGraph<TEdge,TVertex> :: addVertex( TVertex * vertex )
{
  Graph<TEdge,TVertex> :: addVertex( vertex );
}

template <class TEdge,class TVertex>
TEdge * UGraph<TEdge,TVertex> :: findEdge( const TVertex * fromVertex, const TVertex * toVertex ) const
{
  TEdge         * uedge;
  const TVertex * uvertex;

  uvertex = fromVertex;

  uedge = uvertex->getFirstUEdge();

  while( uedge ) {
    if( ( uedge->getUVertex1() == toVertex ) && ( uedge->getUVertex2() == fromVertex ) ||
        ( uedge->getUVertex1() == fromVertex ) && ( uedge->getUVertex2() == toVertex ) ) {
      return( uedge );
    }

    uedge = uedge->getNextUEdge( uvertex );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
void UGraph<TEdge,TVertex> :: removeEdge( TEdge * edge )
{
  TVertex * vertex;
  TEdge   * uedge;

  uedge = edge;

  vertex = uedge->getUVertex1();

  vertex->removeEdge( uedge );

  vertex = uedge->getUVertex2();

  vertex->removeEdge( uedge );

  Graph<TEdge,TVertex> :: removeEdge( uedge );
}

template <class TEdge,class TVertex>
void UGraph<TEdge,TVertex> :: removeVertex( TVertex * vertex )
{
  TVertex * uvertex = vertex;
  TEdge   * uedge   = uvertex->firstEdge;

  while( uedge ) {
    removeEdge( uedge );

    uedge = uvertex->firstEdge;
  }

  Graph<TEdge,TVertex> :: removeVertex( uvertex );
}

template <class TEdge,class TVertex>
void UGraph<TEdge,TVertex> :: deleteAllEdges( void )
{
  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    vertex->clearAllEdges();

    vertex = vertex->getNext();
  }

  Graph<TEdge,TVertex> :: deleteAllEdges();
}

//----------------------- UGraph check staff -------------------------

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: findEdgeInVertex( TEdge * edge ) const
{
  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    if( vertex->IfEdgeInVertex( edge ) ) {
      return( 1 );
    }

    vertex = vertex->getNext();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: findEdgeInGraph( TEdge * edge ) const
{
  TEdge * edge1 = Graph<TEdge,TVertex>::getFirstEdge();

  while( edge1 ) {
    if( edge == edge1 ) {
      return( 1 );
    }

    edge1 = edge1->getNext();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: CheckEdgesInVertex( TVertex * vertex ) const
{
  TEdge * edge = vertex->getFirstUEdge();

  while( edge ) {
    if( !findEdgeInGraph( edge ) ) {
      return( 1 );
    }

    edge = edge->getNextUEdge( vertex );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: CheckEdgesConsistency( void ) const
{
  TEdge * edge = Graph<TEdge,TVertex>::getFirstEdge();

  while( edge ) {
    if( !findEdgeInVertex( edge ) ) {
      return( 1 );
    }

    edge = edge->getNext();
  }

  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    if( CheckEdgesInVertex( vertex ) ) {
      return( 2 );
    }

    vertex = vertex->getNext();
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: getEdgesInNodesCount( void ) const
{
  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();
  int       count  = 0;

  while( vertex ) {
    count = count + vertex->getEdgesCount();

    vertex = vertex->getNext();
  }

  return( count );
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: checkGraph( void ) const
{
  int retCode = Graph<TEdge,TVertex> :: checkGraph();

  if( retCode ) {
    return( retCode );
  }

  if( 2 * Graph<TEdge,TVertex>::getEdgesCount() != getEdgesInNodesCount() ) {
    return( 30 );
  }

  if( CheckEdgesConsistency() ) {
    return( 31 );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
std::ostream & UGraph<TEdge,TVertex> :: Print( std::ostream & s, const int Mod ) const
{
  s << std::endl << "Undirected graph: " << std::endl;

  Graph<TEdge,TVertex> :: Print( s, Mod );

  return( s );
}

//-------------------------------- Algorithms -------------------------------

//================================= findPath ================================

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: recFindPath( TVertex * fromVertex, TVertex * toVertex ) const
{
  if( fromVertex == toVertex ) {
    return( true );
  }

  fromVertex->setVisit( 1 );

  TEdge * edge = fromVertex->getFirstUEdge();

  while( edge ) {
    TVertex * vertex = edge->GetAnotherVertex( fromVertex );

    if( !vertex->getVisit() ) {
      if( recFindPath( vertex, toVertex ) ) {
        return( true );
      }
    }

    edge = edge->getNextUEdge( fromVertex );
  }  

  return( false );
}

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: findPath( TVertex * fromVertex, TVertex * toVertex ) const
{
  Graph<TEdge,TVertex>::clearAllVisitFlags();

  return( recFindPath( fromVertex, toVertex ) );
}

//================================= findHamiltonPath =================================

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: recFindHamiltonPath( TVertex * fromVertex,
                                                   TVertex * toVertex, int Vertices,
                                                   std::list<GraphElem*> & coll ) const
{
  if( fromVertex == toVertex ) {
    if( Vertices != Graph<TEdge,TVertex>::getVerticesCount() - 1 ) {
      if( Vertices ) {
        return( false );
      }
      else {
        coll.push_back( fromVertex );
        return( true );
      }
    }
  }

  fromVertex->setVisit( 1 );

  TEdge * edge = fromVertex->getFirstUEdge();

  while( edge ) {
    TVertex * vertex = edge->GetAnotherVertex( fromVertex );

    if( !vertex->getVisit() ) {
      if( recFindHamiltonPath( vertex, toVertex, Vertices - 1, coll ) ) {
        coll.push_back( edge );
        coll.push_back( fromVertex );
        return( true );
      }
    }

    edge = edge->getNextUEdge( fromVertex );
  }

  fromVertex->clearVisit();

  return( false );
}

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: findHamiltonPath( TVertex * fromVertex,
                                                TVertex * toVertex,
                                                std::list<GraphElem*> & coll ) const
{
  Graph<TEdge,TVertex>::clearAllVisitFlags();

  coll.clear();

  return( recFindHamiltonPath( fromVertex, toVertex, Graph<TEdge,TVertex>::getVerticesCount() - 1, coll ) );
}

//================================= findHamiltonCycle =================================

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: recFindHamiltonCycle( TVertex * fromVertex,
                                                    TVertex * toVertex,
                                                    TEdge * FromEdge, const int Vertices,
                                                    std::list<GraphElem*> & coll ) const
{
  if( fromVertex == toVertex ) {
    if( Vertices ) {
      return( false );
    }
    else {
      coll.push_back( fromVertex );
      return( true );
    }
  }

  fromVertex->setVisit( 1 );

  TEdge * edge = fromVertex->getFirstUEdge();

  while( edge ) {
    if( edge != FromEdge ) {
      TVertex * vertex = edge->GetAnotherVertex( fromVertex );

      if( !vertex->getVisit() ) {
        if( recFindHamiltonCycle( vertex, toVertex, edge, Vertices - 1, coll ) ) {
          coll.push_back( edge );
          coll.push_back( fromVertex );
          return( true );
        }
      }
    }

    edge = edge->getNextUEdge( fromVertex );
  }

  fromVertex->clearVisit();

  return( false );
}

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: findHamiltonCycle( std::list<GraphElem*> & coll ) const
{
  Graph<TEdge,TVertex>::clearAllVisitFlags();

  coll.clear();

  int VerticesCount = Graph<TEdge,TVertex>::getVerticesCount();

  if( VerticesCount > 0 ) {
    TVertex * fromVertex = Graph<TEdge,TVertex>::getFirstVertex();

    if( VerticesCount > 1 ) {
      TEdge * edge = fromVertex->getFirstUEdge();

      while( edge ) {
        TVertex * vertex = edge->GetAnotherVertex( fromVertex );

        if( !vertex->getVisit() ) {
          if( recFindHamiltonCycle( vertex, fromVertex, edge, VerticesCount - 1, coll ) ) {
            coll.push_back( edge );
            coll.push_back( fromVertex );
            return( true );
          }
        }

        edge = edge->getNextUEdge( fromVertex );
      }
    }
    else {
      coll.push_back( fromVertex );      
      return( true );
    }
  }

  return( false );
}

//================================= RankingGraph =================================

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: recRankingGraph( TVertex * fromVertex, TEdge * fromEdge, const int rank ) const
{
  fromVertex->setTmpFlag( rank );

  TEdge * edge = fromVertex->getFirstEdge();

  while( edge ) {
    if( edge != fromEdge ) {
      TVertex * vertex = edge->GetAnotherVertex( fromVertex );

      if( !vertex->getTmpFlag() ) {
        if( recRankingGraph( vertex, edge, rank + 1 ) ) {
          return( 1 );
        }
      }
      else {
        return( 1 );
      }
    }

    edge = edge->getNextEdge( fromVertex );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: rankingGraph( TVertex * fromVertex ) const
{
  Graph<TEdge,TVertex>::clearAllVerticesTmpFlags();

  return( recRankingGraph( fromVertex, 0, 1 ) );
}

//================================= FindComponents =================================

template <class TEdge,class TVertex>
void UGraph<TEdge,TVertex> :: recFindComponents( TVertex * fromVertex, const int component ) const
{
  fromVertex->setTmpFlag( component );

  TEdge * edge = fromVertex->getFirstUEdge();

  while( edge ) {
    TVertex * vertex = edge->GetAnotherVertex( fromVertex );

    if( !vertex->getTmpFlag() ) {
      recFindComponents( vertex, component );
    }

    edge = edge->getNextUEdge( fromVertex );
  }
}

template <class TEdge,class TVertex>
int UGraph<TEdge,TVertex> :: findComponents( void ) const
{
  Graph<TEdge,TVertex>::clearAllVerticesTmpFlags();

  int component = 0;

  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    if( !vertex->getTmpFlag() ) {
      component++;

      recFindComponents( vertex, component );
    }

    vertex = vertex->getNext();
  }

  return( component ) ; 
}

//================================= findEulerPath =================================

template <class TEdge,class TVertex>
TEdge * UGraph<TEdge,TVertex> :: getNextExistingEdge( TVertex * vertex ) const
{
  TEdge * edge = vertex->getFirstUEdge();

  while( edge ) {
    if( !edge->getTmpFlag() ) {
      return( edge );
    }

    edge = edge->getNextUEdge( vertex );
  }

  return( 0 );
}

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: ifEulerPathExists( TVertex * fromVertex,
                                                 TVertex * toVertex ) const
{
  int edges = fromVertex->getEdgesCount() + toVertex->getEdgesCount();

  if( edges % 2 ) {
    return( false );
  }

  TVertex * vertex = Graph<TEdge,TVertex>::getFirstVertex();

  while( vertex ) {
    if( ( vertex != fromVertex ) && ( vertex != toVertex ) ) {
      if( vertex->getEdgesCount() % 2 ) {
        return( false );
      }
    }

    vertex = vertex->getNext();
  }

  return( true );
}

template <class TEdge,class TVertex>
TVertex * UGraph<TEdge,TVertex> :: recFindEulerPath( TVertex * vertex, std::stack<TVertex*> & VStack ) const
{
  TEdge   * edge;
  TVertex * newVertex;

  for(; ( edge = getNextExistingEdge( vertex ) ) != 0; vertex = newVertex ) {
    VStack.push( vertex ); 

    newVertex = edge->GetAnotherVertex( vertex );

    edge->setTmpFlag( 1 );
  }

  return( vertex );
}

template <class TEdge,class TVertex>
bool UGraph<TEdge,TVertex> :: findEulerPath( TVertex * fromVertex,
                                             TVertex * toVertex,
                                             std::list<TVertex*> & coll ) const
{
  std::stack<TVertex*> VStack;
  bool                 RetCode = true;

  if( findComponents() == 1 ) {
    if( ifEulerPathExists( fromVertex, toVertex ) ) {
      Graph<TEdge,TVertex>::clearAllEdgesTmpFlags();

      coll.clear();

      coll.push_back( toVertex );

      if( recFindEulerPath( fromVertex, VStack ) == toVertex ) {
        while( !VStack.empty() ) {
          TVertex * vertex = VStack.top();

          VStack.pop();

          coll.push_back( vertex );

          if( recFindEulerPath( vertex, VStack ) != vertex ) {
            RetCode = false;
            break;
          }
        }
      }
      else {
        RetCode = false;
      }
    }
    else {
      RetCode = false;
    }
  }
  else {
    RetCode = false;
  }

  return( RetCode );
}

}}

//---------------------------- End of file ---------------------------
